<?php

namespace Drupal\files_url_replacer;

use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\StreamWrapper\StreamWrapperManager;
use Drupal\Core\Url;

/**
 * Replace baseurl.
 */
class CustomFileUrlGenerator extends FileUrlGenerator {

  /**
   * {@inheritdoc}
   */
  public function generateString(string $uri) : string {
    $string = parent::generateString($uri);
    $active = &drupal_static(__FUNCTION__);
    if (!isset($active)) {
      $active = \Drupal::config('files_url_replacer.settings')->get('active');
    }
    $ext = pathinfo($uri, PATHINFO_EXTENSION);
    $exclude = ['css', 'js'];

    if ($string && $active && !in_array($ext, $exclude) && StreamWrapperManager::getScheme($uri) == 'public') {
      $url = \Drupal::config('files_url_replacer.settings')->get('url');
      if (!empty($url) && $this->checkIfExists($uri)) {
        $httphost = rtrim(\Drupal::request()->getSchemeAndHttpHost(), '/');
        $baseurl = rtrim(Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString(), '/');
        if ($httphost != $baseurl) {
          $site_path = str_replace($httphost, '', $baseurl);
          $string = str_replace($site_path, '', $string);
        }
        $string = rtrim($url, '/') . $string;
      }
    }

    return $string;
  }

  /**
   * {@inheritdoc}
   */
  public function generateAbsoluteString(string $uri) : string {
    $string = parent::generateAbsoluteString($uri);
    if (!isset($active)) {
      $active = \Drupal::config('files_url_replacer.settings')->get('active');
    }
    $ext = pathinfo($uri, PATHINFO_EXTENSION);
    $exclude = ['css', 'js'];

    if ($string && $active && !in_array($ext, $exclude) && StreamWrapperManager::getScheme($uri) == 'public') {
      $url = \Drupal::config('files_url_replacer.settings')->get('url');
      if (!empty($url) && $this->checkIfExists($uri)) {
        $baseurl = rtrim(Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString(), '/');
        $string = str_replace($baseurl, rtrim($url, '/'), $string);
      }
    }

    return $string;
  }

  /**
   * {@inheritdoc}
   */
  private function checkIfExists(string $uri) : bool {
    $check = &drupal_static(__FUNCTION__);
    if (!isset($check)) {
      $check = \Drupal::config('files_url_replacer.settings')->get('check');
    }

    if ($check == TRUE && file_exists($uri)) {
      return FALSE;
    }

    return TRUE;
  }

}
